let chaininfo = require("./chaininfo.json");
const { Api, JsonRpc, RpcError } = require("eosjs");
const { JsSignatureProvider } = require("eosjs/dist/eosjs-jssig");
const signatureProvider = new JsSignatureProvider([]);
const fetch = require("isomorphic-fetch");
const { TextEncoder, TextDecoder } = require("text-encoding");

const { createClient } = require("@liquidapps/dapp-client");
const Hashids = require("hashids/cjs");
const axios = require("axios").default;
const encoding = [
  "a",
  "b",
  "c",
  "d",
  "e",
  "f",
  "g",
  "h",
  "i",
  "j",
  "k",
  "l",
  "m",
  "n",
  "o",
];
Object.defineProperty(exports, "__esModule", { value: true });

exports.reguser = async (env, email, contact, otp, prvkey) => {
  try {
    const service = await getservice(env);
    const vacc = await this.converttovacc(email);
    let resp = await service.push_liquid_account_transaction(
      //sign_liquid_account_transaction(
      chaininfo[env].contract,
      prvkey,
      "regboth",
      {
        vaccount: vacc,
        email: email,
        contact: contact,
        otp: otp,
        payload: {
          vaccount: vacc,
        },
      }
    );
    // let pushresult = await pushTransaction(
    //   env,
    //   resp.public_key,
    //   resp.signature,
    //   resp.payload,
    //   []
    // );
    // if (pushresult) {
    //   return { accname: vacc };
    // }
    return { accname: vacc };
  } catch (err) {
    throw new Error(err);
  }
};

exports.recoveruser = async (env, email, otp, prvkey) => {
  try {
    const service = await getservice(env);
    const dappacc = await this.converttovacc(email);
    let resp = await service.push_liquid_account_transaction(
      //sign_liquid_account_transaction(
      chaininfo[env].contract,
      prvkey,
      "confirmcode",
      {
        vaccount: dappacc,
        otp: otp,
        payload: {
          vaccount: dappacc,
        },
      }
    );
    // let pushresult = await pushTransaction(
    //   env,
    //   resp.public_key,
    //   resp.signature,
    //   resp.payload,
    //   []
    // );
    // if (pushresult) {
    //   return { accname: dappacc };
    // }
    return { accname: dappacc };
  } catch (err) {
    throw new Error(err);
  }
};

async function gettrxbyid(env, trxid) {
  const dspEndpt = chaininfo[env].gettrx;
  const rpc = new JsonRpc(dspEndpt, { fetch });
  const api = new Api({
    rpc,
    signatureProvider,
    textDecoder: new TextDecoder(),
    textEncoder: new TextEncoder(),
  });
  let resp = await rpc.history_get_transaction(trxid);
  return resp;
}

async function getCorrectTransactionId(env, transactionIds) {
  const requests = transactionIds.map((v) => gettrxbyid(env, v));
  const responses = await axios.all(requests);
  let index = -1;

  for (let i = 0; i < responses.length; i++) {
    if (responses[i].error) {
      continue;
    } else {
      index = i;
      break;
    }
  }

  if (index >= 0) {
    return transactionIds[index];
  }

  return "";
}

async function pushTransaction(env, pubkey, signature, payload, txIds) {
  if (txIds.length) {
    const txId = await getCorrectTransactionId(env, txIds);
    if (txId.length > 0) {
      return true;
    }
  }

  try {
    const service = await getservice(env);
    const response = await service.post_liquid_account_transaction(
      chaininfo[env].contract,
      pubkey,
      signature,
      payload
    );
    const txId = response.result.transaction_id;
    txIds.push(txId);
    return pushTransaction(pubkey, signature, payload, txIds);
  } catch (e) {
    const error = JSON.stringify(e);
    if (
      error.includes("invalid nonce") ||
      error.includes("Provided public key already registered for this") ||
      error.includes("Call updateacckey to add keys")
    ) {
      return true;
    } else {
      throw new Error(e);
    }
  }
}
async function getservice(env) {
  let client = await createClient({
    network: "kylin",
    httpEndpoint: chaininfo[env].endpoint,
    fetch: fetch,
  });
  const service = await client.service("vaccounts", chaininfo[env].contract);
  return service;
}

exports.converttovacc = async (email) => {
  const hashids = new Hashids(email, 9);
  let hashsh = hashids.encode(999);
  let accname = "dap" + (await getaccname(hashsh.toLowerCase()));
  return accname;
};

async function getaccname(data) {
  let newdata = "";
  for (let i = 0; i < data.length; i += 1) {
    if (data.charAt(i) === 0 || data.charAt(i) > 5 || data.charAt(i) === "0") {
      newdata += encoding[data.charAt(i)];
    } else newdata += data.charAt(i);
  }
  return newdata;
}
